@@ -99,6 +99,7 @@ class Table:
         self._properties = dict()
         self._columns = None
         self._dataLines = []
+        self._has_none = None
         
     def Unserialize(self, bytesStream, oldSerialize):
         self._version = bytesStream.read(1)[0]
@@ -109,16 +110,18 @@ class Table:
         self._properties = Properties.Unserialize(bytesStream)
         self._columns = ColumnSet()
         self._columns.Unserialize(bytesStream)
+        self._has_none = [False] * len(self._columns._columns)
         
         count = struct.unpack("h", bytesStream.read(2))[0]      
         for element in range(count):    
             dataLine = DataLine(self._columns)
             dataLine.Unserialize(bytesStream)
+            self._has_none = [a or b for a,b in zip(self._has_none,dataLine._has_none)]
             self.Add(dataLine)
             
     def Add(self, dataLine):
         if self._columns == None or len(self._columns._columns) <= 0:
-            raise Exception ("You can't add a data line to a table without assigning first a column to the table.")
+            raise Exception ("You can't add a data line to a table without first assigning a column to the table.")
             return
         self._dataLines.append(dataLine)
       
@@ -140,26 +143,29 @@ class Column:
     def __init__(self,name,col_type):
         self._name = name
         self._type = col_type
+        self._has_none = False
 
 class DataLine:
     def __init__(self, columns):
         self._columns = columns
         self._dataLine = [None] * len(columns._columns)
+        self._has_none = [False] * len(columns._columns);
         
     def Unserialize(self,bytesStream):
         count = _read7BitEncodedInt(bytesStream)
-        sortedList = OrderedDict()
+        missing_values = OrderedDict()
         num2 = 0
         for element in range(count):
             num3 = _read7BitEncodedInt(bytesStream)
             num2 += num3
-            sortedList[num2] = num3 
+            missing_values[num2] = num3 
             
         supportList = list(self._columns._columns.values())
         
         for element in range(len(self._columns._columns)):
-            if element in sortedList:
+            if element in missing_values:
                 self._dataLine[element] = None
+                self._has_none[element] = True
             else:
                 self._dataLine[element] = _readTypeValue(supportList[element]._type, bytesStream)
                 
@@ -206,8 +212,6 @@ class Collection:
             else:
                 self._arrayList['Loop'] = 'Internal'
                 
-            
-            
 class Bag:
     def __init__(self):
         self.Key = str()
@@ -272,6 +276,25 @@ def _getDatatype (dataTypeEnum):
     }
     return thisdict[dataTypeEnum]
 
+def _getCDataType (dataTypeEnum):
+    thisdict = {
+        5:'SByte',
+        6:'Byte',
+        7:'Int16',
+        8:'UInt16',
+        9:'Int32',
+        10:'UInt32',
+        11:'Int64',
+        12:'UInt64',
+        4:'Char',
+        13:'Single',
+        14:'Double',
+        3:'Boolean',
+        15:'Decimal',
+        16:'DateTime',
+        }
+    return thisdict[dataTypeEnum]
+
 def _readTypeValue(dataTypeEnum, bytesStream):
     dataType, size = _getDatatype(dataTypeEnum)
     if dataType == "s":#string
@@ -546,7 +569,7 @@ def _processVersionN(version, composite, logData):
     
     return logData
 
-def binStreamToDF(file, logData=LogData()):
+def binStreamToDF(file, logData=LogData(), null_promoting={}):
     """
     Unpacks binary file stream into LogData
     
@@ -557,13 +580,18 @@ def binStreamToDF(file, logData=LogData()):
        
     bin_file : bool
         whether source file is .bin extension
+        
+    null_promoting : dict
+        A dictionary with a .NET Source Type key and a value of either one of 
+        the following (default, object, float, Int64, string, error).\n
 
     Returns
     -------
     LogData
         Structure containing most file data
 
-    """   
+    """  
+    
     #header
     TAGS = file.read(4)
     if TAGS != b'TAGS':
@@ -628,7 +656,38 @@ def binStreamToDF(file, logData=LogData()):
                 data = []
                 for dataline in frame._baseframe._dataLines:
                     data.append(dataline._dataLine)
-                dFrame = pd.DataFrame(data, columns=frame._baseframe._columns._columns.keys())
+                    
+                if any(frame._baseframe._has_none): ##Null handling
+                    #Transpose data array
+                    data = list(map(list, zip(*data)))
+                    series = []
+                    dFrame = pd.DataFrame()
+                    #Iterate through columns and handle null values accordingly
+                    for col,has_none,col_data in zip(frame._baseframe._columns._columns.values(),frame._baseframe._has_none,data ):
+                        #Create a pandas series for each column
+                        if (has_none):
+                            # Lookup desired handling
+                            type_str = _getCDataType(col._type)
+                            if type_str in null_promoting:
+                                desired_type = null_promoting[type_str]
+                            else:
+                                series.append(pd.Series(col_data, name=col._name))
+                                continue;
+                                
+                            if desired_type == 'default':
+                                series.append(pd.Series(col_data, name=col._name))
+                            elif desired_type == 'error':
+                                raise Exception(F"Null values encountered in column {col._name} of type {type_str}")
+                            else:
+                                series.append(pd.Series(col_data, name=col._name, dtype=desired_type))
+                        else:     
+                            series.append(pd.Series(col_data, name=col._name))
+                        
+                    dFrame = pd.concat(series, axis=1)
+                
+                else: #Default dataframe construction
+                    dFrame = pd.DataFrame(data, columns=frame._baseframe._columns._columns.keys())
+                    
                 logData.dataFrame = dFrame
                 logData.properties = frame._baseframe._properties
                 
@@ -678,9 +737,7 @@ def getCollection(frame, name):
             
     return series
         
-        
-
-def binFileToDF(path, extension=None):
+def binFileToDF(path, extension=None, null_promoting={}):
     """
     Unpacks binary file into LogData
     
@@ -692,6 +749,36 @@ def binFileToDF(path, extension=None):
     extension : str
         Assume given file extension. ex: 'bin'
         
+    null_promoting : dict
+        A dictionary with a .NET Source Type key and a value of either one of 
+        the following (default, object, float, Int64, string, error).\n
+        
+        The possible dictionary keys are the .NET simple types:
+            * SByte
+            * Byte
+            * Int16
+            * UInt16
+            * Int32
+            * UInt32
+            * Int64
+            * UInt64
+            * Char
+            * Single
+            * Double
+            * Boolean
+            * Decimal
+            * DateTime
+            
+        This dictionary values determines how null values in deserialization affect 
+        the resulting LogData dataframe column:
+            
+            * default: use pandas automatic inference when dealing with null values on a column
+            * object: The returned type is the generic python object type
+            * float: The returned type is the python float type
+            * Int64: The returned type is the pandas Nullable Integer Int64 type
+            * string: Values are returned as strings
+            * error: Raises and exception when null values are encountered 
+            
     Returns
     -------
     LogData
@@ -716,7 +803,7 @@ def binFileToDF(path, extension=None):
     else:
         logData._fileType = extension
     
-    logData = binStreamToDF(file, logData)
+    logData = binStreamToDF(file, logData, null_promoting)
     
     file.close()
-    return logData
+    return logData    
\ No newline at end of file
